@@ -12,8 +12,6 @@ from compas.geometry import offset_polygon
 from compas.utilities import iterable_like
 from compas.utilities import pairwise
 
-from compas.datastructures.mesh.core import BaseMesh
-
 
 __all__ = [
     'mesh_subdivide',
@@ -27,11 +25,50 @@ __all__ = [
 ]
 
 
+def subd_factory(cls):
+
+    class SubdMesh(cls):
+
+        _add_vertex = cls.add_vertex
+        _add_face = cls.add_face
+        _insert_vertex = cls.insert_vertex
+
+        def add_vertex(self, x, y, z):
+            key = self._max_vertex = self._max_vertex + 1
+            if key not in self.vertex:
+                self.vertex[key] = {}
+                self.halfedge[key] = {}
+            self.vertex[key] = dict(x=x, y=y, z=z)
+            return key
+
+        def add_face(self, vertices):
+            fkey = self._max_face = self._max_face + 1
+            self.face[fkey] = vertices
+            self.facedata[fkey] = {}
+            for i in range(-1, len(vertices) - 1):
+                u = vertices[i]
+                v = vertices[i + 1]
+                self.halfedge[u][v] = fkey
+                if u not in self.halfedge[v]:
+                    self.halfedge[v][u] = None
+            return fkey
+
+        def insert_vertex(self, fkey):
+            x, y, z = self.face_center(fkey)
+            w = self.add_vertex(x=x, y=y, z=z)
+            for u, v in self.face_halfedges(fkey):
+                self.add_face([u, v, w])
+            del self.face[fkey]
+            return w
+
+    return SubdMesh
+
+
 def mesh_fast_copy(other):
+    SubdMesh = subd_factory(type(other))
     subd = SubdMesh()
     subd.vertex = deepcopy(other.vertex)
     subd.face = deepcopy(other.face)
-    # subd.edgedata = deepcopy(other.edgedata)
     subd.facedata = deepcopy(other.facedata)
     subd.halfedge = deepcopy(other.halfedge)
     subd._max_vertex = other._max_vertex
@@ -39,50 +76,6 @@ def mesh_fast_copy(other):
     return subd
 
 
-class SubdMesh(BaseMesh):
-
-    from compas.datastructures.mesh.core import mesh_split_edge
-
-    _add_vertex = BaseMesh.add_vertex
-    _add_face = BaseMesh.add_face
-    _insert_vertex = BaseMesh.insert_vertex
-
-    split_edge = mesh_split_edge
-
-    def add_vertex(self, x, y, z):
-        key = self._max_vertex = self._max_vertex + 1
-
-        if key not in self.vertex:
-            self.vertex[key] = {}
-            self.halfedge[key] = {}
-
-        self.vertex[key] = dict(x=x, y=y, z=z)
-
-        return key
-
-    def add_face(self, vertices):
-        fkey = self._max_face = self._max_face + 1
-
-        self.face[fkey] = vertices
-        self.facedata[fkey] = {}
-
-        for i in range(-1, len(vertices) - 1):
-            u = vertices[i]
-            v = vertices[i + 1]
-            self.halfedge[u][v] = fkey
-            if u not in self.halfedge[v]:
-                self.halfedge[v][u] = None
-
-        return fkey
-
-    def insert_vertex(self, fkey):
-        x, y, z = self.face_center(fkey)
-        w = self.add_vertex(x=x, y=y, z=z)
-        for u, v in self.face_halfedges(fkey):
-            self.add_face([u, v, w])
-        del self.face[fkey]
-        return w
-
 # distinguish between subd of meshes with and without boundary
 # closed vs. open
 # pay attention to extraordinary points
@@ -98,17 +91,16 @@ def mesh_subdivide(mesh, scheme='catmullclark', **options):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         A mesh object.
-    scheme : {'tri', 'quad', 'corner', 'catmullclark', 'doosabin', 'frames', 'loop'}, optional
+    scheme : Literal['tri', 'quad', 'corner', 'catmullclark', 'doosabin', 'frames', 'loop'], optional
         The scheme according to which the mesh should be subdivided.
-        Default is ``'catmullclark'``.
-    options : dict
+    **options : dict[str, Any], optional
         Optional additional keyword arguments.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         The subdivided mesh.
 
     Raises
@@ -140,14 +132,14 @@ def mesh_subdivide_tri(mesh, k=1):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object that will be subdivided.
-    k : int
-        Optional. The number of levels of subdivision. Default is ``1``.
+    k : int, optional
+        The number of levels of subdivision.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
     Examples
@@ -180,14 +172,14 @@ def mesh_subdivide_quad(mesh, k=1):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object that will be subdivided.
-    k : int
-        Optional. The number of levels of subdivision. Default is ``1``.
+    k : int, optional
+        The number of levels of subdivision.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
     Examples
@@ -236,20 +228,19 @@ def mesh_subdivide_corner(mesh, k=1):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object that will be subdivided.
-    k : int
-        Optional. The number of levels of subdivision. Default is ``1``.
+    k : int, optional
+        The number of levels of subdivision.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
     Notes
     -----
-    This is essentially the same as Loop subdivision, but applied to general
-    meshes.
+    This is essentially the same as Loop subdivision, but applied to general meshes.
 
     """
     cls = type(mesh)
@@ -280,16 +271,16 @@ def mesh_subdivide_catmullclark(mesh, k=1, fixed=None):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object that will be subdivided.
-    k : int
-        Optional. The number of levels of subdivision. Default is ``1``.
-    fixed : list
-        Optional. A list of fixed vertices. Default is ``None``.
+    k : int, optional
+        The number of levels of subdivision.
+    fixed : list[int], optional
+        A list of fixed vertices.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
     Notes
@@ -298,6 +289,13 @@ def mesh_subdivide_catmullclark(mesh, k=1, fixed=None):
     smoothing after every level of further subdivision. Smoothing is done
     according to the scheme prescribed by the Catmull-Clark algorithm.
 
+    References
+    ----------
+    .. [1] Tony DeRose, Michael Kass and Tien Truong.
+           Subdivision Surfaces in Character Animation.
+           Pixar Animation Studios.
+           see https://graphics.pixar.com/library/Geri/paper.pdf
+
     Examples
     --------
     >>> from compas.geometry import Box
@@ -315,11 +313,11 @@ def mesh_subdivide_catmullclark(mesh, k=1, fixed=None):
 
     The algorithm supports "integer creasing" as described in
     Subdivision Surfaces in Character Animation [1]_.
-    Creases are supported through the optional edge attribute ``'crease'``,
+    Creases are supported through the optional edge attribute "crease",
     which can be set to an integer value that defines how sharp the crease is wrt
     the number of subdivision steps.
 
-    To add an infinitely sharp crease to an edge, set the ``'crease'`` attribute of the edge
+    To add an infinitely sharp crease to an edge, set the "crease" attribute of the edge
     to a number higher than the number of subdivision steps.
 
     >>> from compas.geometry import Box, dot_vectors
@@ -332,15 +330,9 @@ def mesh_subdivide_catmullclark(mesh, k=1, fixed=None):
 
     >>> subd = cage.subdivide(k=4)
 
-    References
-    ----------
-    .. [1] Tony DeRose, Michael Kass and Tien Truong.
-           Subdivision Surfaces in Character Animation.
-           Pixar Animation Studios.
-           see https://graphics.pixar.com/library/Geri/paper.pdf
-
     """
     cls = type(mesh)
+
     if not fixed:
         fixed = []
     fixed = set(fixed)
@@ -463,16 +455,16 @@ def mesh_subdivide_doosabin(mesh, k=1, fixed=None):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object that will be subdivided.
-    k : int
-        Optional. The number of levels of subdivision. Default is ``1``.
-    fixed : list
-        Optional. A list of fixed vertices. Default is ``None``.
+    k : int, optional
+        The number of levels of subdivision.
+    fixed : list[int], optional
+        A list of fixed vertices.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
     Examples
@@ -495,6 +487,7 @@ def mesh_subdivide_doosabin(mesh, k=1, fixed=None):
     fixed = set(fixed)
 
     cls = type(mesh)
+    SubdMesh = subd_factory(cls)
 
     for _ in range(k):
         old_xyz = {key: mesh.vertex_coordinates(key) for key in mesh.vertices()}
@@ -577,26 +570,23 @@ def mesh_subdivide_frames(mesh, offset, add_windows=False):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object to be subdivided.
-    offset : float or dict
+    offset : float | dict[int, float]
         The offset distance to create the frames.
         A single value will result in a constant offset everywhere.
-        A dictionary mapping facekey: offset will be processed accordingly.
-    add_windows : boolean
-        Optional. Flag to add window face. Default is ``False``.
+        A dictionary mapping faces to offset values will be processed accordingly.
+    add_windows : bool, optional
+        If True, add a window face in the frame opening.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
-    Examples
-    --------
-    >>>
-
     """
     cls = type(mesh)
+    SubdMesh = subd_factory(cls)
 
     subd = SubdMesh()
 
@@ -647,16 +637,16 @@ def trimesh_subdivide_loop(mesh, k=1, fixed=None):
 
     Parameters
     ----------
-    mesh : Mesh
+    mesh : :class:`~compas.datastructures.Mesh`
         The mesh object that will be subdivided.
-    k : int
-        Optional. The number of levels of subdivision. Default is ``1``.
-    fixed : list
-        Optional. A list of fixed vertices. Default is ``None``.
+    k : int, optional
+        The number of levels of subdivision.
+    fixed : list[int], optional
+        A list of fixed vertices.
 
     Returns
     -------
-    Mesh
+    :class:`~compas.datastructures.Mesh`
         A new subdivided mesh.
 
     Examples
