@@ -13,6 +13,8 @@ from compas_rhino.utilities import create_layers_from_path
 from compas_rhino.utilities import clear_layer
 from compas_rhino.utilities import clear_current_layer
 
+import System
+
 from System.Collections.Generic import List
 from System.Drawing.Color import FromArgb
 from System.Enum import ToObject
@@ -56,6 +58,7 @@ add_brep = sc.doc.Objects.AddBrep
 add_sphere = sc.doc.Objects.AddSphere
 add_mesh = sc.doc.Objects.AddMesh
 add_circle = sc.doc.Objects.AddCircle
+add_surface = sc.doc.Objects.AddSurface
 
 TOL = sc.doc.ModelAbsoluteTolerance
 
@@ -78,6 +81,7 @@ __all__ = [
     'draw_spheres',
     'draw_mesh',
     'draw_circles',
+    'draw_surfaces',
 ]
 
 
@@ -113,12 +117,13 @@ def draw_labels(labels, **kwargs):
 
     Parameters
     ----------
-    labels : list of dict
+    labels : list[dict]
         A list of labels dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -177,12 +182,13 @@ def draw_points(points, **kwargs):
 
     Parameters
     ----------
-    labels : list of dict
+    labels : list[dict]
         A list of point dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -233,12 +239,13 @@ def draw_lines(lines, **kwargs):
 
     Parameters
     ----------
-    labels : list of dict
+    labels : list[dict]
         A list of line dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -302,12 +309,13 @@ def draw_geodesics(geodesics, **kwargs):
 
     Parameters
     ----------
-    labels : list of dict
+    labels : list[dict]
         A list of geodesic dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -369,12 +377,13 @@ def draw_polylines(polylines, **kwargs):
 
     Parameters
     ----------
-    labels : list of dict
+    labels : list[dict]
         A list of polyline dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -427,34 +436,25 @@ def draw_polylines(polylines, **kwargs):
 
 
 @wrap_drawfunc
-def draw_breps(faces, srf=None, u=10, v=10, trim=True, tangency=True, spacing=0.1, flex=1.0, pull=1.0, join=False, **kwargs):
+def draw_breps(faces, u=10, v=10, join=False, **kwargs):
     """Draw polygonal faces as Breps, and optionally set individual name, color,
     and layer properties.
 
     Parameters
     ----------
-    faces : list of dict
+    faces : list[dict]
         A list of brep dictionaries.
-    srf : GUID, optional
-        A target surface.
+        See Notes, for more information about the structure of the dict.
     u : int, optional
-        Default is 10.
+        Number of spans in the U direction.
     v : int, optional
-        Default is 10.
-
-    Other Parameters
-    ----------------
-    trim : bool, optional
-    tangency : bool, optional
-    spacing : float, optional
-    flex : float, optional
-    pull : float, optional
+        Number of spans in the V direction.
     join : bool, optional
-        Join the individual faces as polysurfaces. Default is False.
+        If True, join the individual faces as polysurfaces
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -550,13 +550,15 @@ def draw_cylinders(cylinders, cap=False, **kwargs):
 
     Parameters
     ----------
-    cylinders : list of dict
+    cylinders : list[dict]
         A list of cylinder dictionaries.
+        See Notes, for more information about the structure of the dict.
     cap : bool, optional
+        If True, add caps.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -624,8 +626,9 @@ def draw_pipes(pipes, cap=2, fit=1.0, **kwargs):
 
     Parameters
     ----------
-    pipes : list of dict
+    pipes : list[dict]
         A list of pipe dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Other Parameters
     ----------------
@@ -634,7 +637,7 @@ def draw_pipes(pipes, cap=2, fit=1.0, **kwargs):
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -697,12 +700,13 @@ def draw_spheres(spheres, **kwargs):
 
     Parameters
     ----------
-    spheres : list of dict
+    spheres : list[dict]
         A list of sphere dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -756,19 +760,18 @@ def draw_mesh(vertices, faces, name=None, color=None, disjoint=False, **kwargs):
 
     Parameters
     ----------
-    vertices : :obj:`list` of point
+    vertices : list[[float, float, float] | :class:`~compas.geometry.Point`]
         A list of point locations.
-    faces : :obj:`list` of :obj:`list` of :obj:`int`
-        A list of faces as lists of indices into ``vertices``.
-    name : :obj:`str`, optional
-    color : RGB :obj:`tuple`, optional
-    disjoint : :obj:`bool`, optional
-        Draw the mesh with disjoint faces.
-        Default is ``False``.
+    faces : list[list[int]]
+        A list of faces as lists of indices into `vertices`.
+    name : str, optional
+    color : tuple[[int, int, int]], optional
+    disjoint : bool, optional
+        If True, draw the mesh with disjoint faces.
 
     Returns
     -------
-    str or GUID
+    System.Guid
 
     """
     mesh = RhinoMesh()
@@ -824,19 +827,22 @@ def draw_mesh(vertices, faces, name=None, color=None, disjoint=False, **kwargs):
 
     mesh.Normals.ComputeNormals()
     mesh.Compact()
+
     guid = add_mesh(mesh)
-    if guid:
+
+    if guid != System.Guid.Empty:
         obj = find_object(guid)
-        attr = obj.Attributes
-        if color:
-            attr.ObjectColor = FromArgb(*color)
-            attr.ColorSource = ColorFromObject
-        else:
-            attr.ColorSource = ColorFromLayer
-        if name:
-            attr.Name = name
-        obj.CommitChanges()
-    return guid
+        if obj:
+            attr = obj.Attributes
+            if color:
+                attr.ObjectColor = FromArgb(*color)
+                attr.ColorSource = ColorFromObject
+            else:
+                attr.ColorSource = ColorFromLayer
+            if name:
+                attr.Name = name
+            obj.CommitChanges()
+        return guid
 
 
 @wrap_drawfunc
@@ -845,12 +851,13 @@ def draw_faces(faces, **kwargs):
 
     Parameters
     ----------
-    faces : list of dict
+    faces : list[dict]
         A list of face dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -882,13 +889,7 @@ def draw_faces(faces, **kwargs):
         elif v == 4:
             mfaces = [[0, 1, 2, 3]]
         else:
-            mfaces = _face_to_max_quad(points, range(v))
-            if vertexcolors:
-                r, g, b = [sum(component) / v for component in zip(*vertexcolors)]
-                r = int(min(max(0, r), 255))
-                g = int(min(max(0, g), 255))
-                b = int(min(max(0, b), 255))
-                vertexcolors.append((r, g, b))
+            mfaces = [list(range(v))]
 
         guid = draw_mesh(points, mfaces, color=color, name=name, clear=False, redraw=False, layer=None)
 
@@ -920,12 +921,13 @@ def draw_circles(circles, **kwargs):
 
     Parameters
     ----------
-    circles : list of dict
+    circles : list[dict]
         A list of circle dictionaries.
+        See Notes, for more information about the structure of the dict.
 
     Returns
     -------
-    list of GUID
+    list[System.Guid]
 
     Notes
     -----
@@ -971,3 +973,115 @@ def draw_circles(circles, **kwargs):
         obj.CommitChanges()
         guids.append(guid)
     return guids
+
+
+@wrap_drawfunc
+def draw_curves(curves, **kwargs):
+    """Draw curves and optionally set individual name, color, and layer properties.
+
+    Parameters
+    ----------
+    curves : list[dict]
+        A list of curve dictionaries.
+        See Notes, for more information about the structure of the dict.
+
+    Returns
+    -------
+    list[System.Guid]
+
+    Notes
+    -----
+    A curve dict has the following schema:
+
+    .. code-block:: python
+
+        Schema({
+            'curve': compas.geometry.Curve,
+            Optional('name', default=''): str,
+            Optional('color', default=None): And(lambda x: len(x) == 3, all(0 <= y <= 255 for y in x)),
+            Optional('layer', default=None): str
+        })
+
+    """
+    guids = []
+    for data in iter(curves):
+        curve = data['curve']
+        name = data.get('name', '')
+        color = data.get('color')
+        layer = data.get('layer')
+        guid = add_curve(curve.rhino_curve)
+        if not guid:
+            continue
+        obj = find_object(guid)
+        if not obj:
+            continue
+        attr = obj.Attributes
+        if color:
+            attr.ObjectColor = FromArgb(*color)
+            attr.ColorSource = ColorFromObject
+        else:
+            attr.ColorSource = ColorFromLayer
+        if layer and find_layer_by_fullpath:
+            index = find_layer_by_fullpath(layer, True)
+            if index >= 0:
+                attr.LayerIndex = index
+        attr.Name = name
+        obj.CommitChanges()
+        guids.append(guid)
+    return guids
+
+
+@wrap_drawfunc
+def draw_surfaces(surfaces, **kwargs):
+    """Draw surfaces and optionally set individual name, color, and layer properties.
+
+    Parameters
+    ----------
+    surfaces : list[dict]
+        A list of surface dictionaries.
+        See Notes, for more information about the structure of the dict.
+
+    Returns
+    -------
+    list[System.Guid]
+
+    Notes
+    -----
+    A surface dict has the following schema:
+
+    .. code-block:: python
+
+        Schema({
+            'surface': compas.geometry.Surface,
+            Optional('name', default=''): str,
+            Optional('color', default=None): And(lambda x: len(x) == 3, all(0 <= y <= 255 for y in x)),
+            Optional('layer', default=None): str
+        })
+
+    """
+    guids = []
+    for data in iter(surfaces):
+        surface = data['surface']
+        name = data.get('name', '')
+        color = data.get('color')
+        layer = data.get('layer')
+        guid = add_surface(surface.rhino_surface)
+        if not guid:
+            continue
+        obj = find_object(guid)
+        if not obj:
+            continue
+        attr = obj.Attributes
+        if color:
+            attr.ObjectColor = FromArgb(*color)
+            attr.ColorSource = ColorFromObject
+        else:
+            attr.ColorSource = ColorFromLayer
+        if layer and find_layer_by_fullpath:
+            index = find_layer_by_fullpath(layer, True)
+            if index >= 0:
+                attr.LayerIndex = index
+        attr.Name = name
+        obj.CommitChanges()
+        guids.append(guid)
+    return guids
